#include <GpioRead.h>
#include <GpioWrite.h>
#include <PwmWrite.h>
#include <AnalogRead.h>
#include <Hysteresis.h>
#include <SoundWrite.h>
#include <Timer.h>
#include <Math.h>
#include <Filter.h>
#include <Delay.h>

int main()
{
	GpioRead buttonSlider = GpioRead (19, true, 10);
	GpioRead buttonPhoto = GpioRead (20, true, 10);
	AnalogRead potentiometer = AnalogRead (15);
	GpioWrite shutter = GpioWrite (18);
	PwmWrite servo = PwmWrite (1);
	GpioWrite led = GpioWrite (12);
	AnalogRead voltageBattery = AnalogRead (16);
	bool lowVoltageBattery = false;
	Filter filterVoltageBattery = Filter (500, true);
	Hysteresis hysteresisVoltageBattery = Hysteresis();
	Delay delayLowVoltageBattery = Delay (250, true);
	Timer time = Timer();
	unsigned long long timeStartSlider = 0;
	unsigned long long timeStartPhoto = 0;
	unsigned long long timePhoto = 0;
	unsigned long long timeIdle = 0;
	bool startSlider = false;
	bool startPhoto = false;
	bool slider = false;
	bool photo = false;
	unsigned long countPhoto = 0;
	Filter filterPotentiometer = Filter (1000, true);
	const unsigned int SETUP_INTERVAL = 1000;
	const unsigned char SETUP_FPS = 24;
	const unsigned int SETUP_CENTER_SERVO = 1500;
	const unsigned int SETUP_TRAVEL_SERVO = 100;
	const float SETUP_LOW_VOLTAGE_BATTERY = 6;
	
	SoundWrite::pin (2);
	PwmWrite::start (50);
	time.start (0);
	
	while (true)
	{
		time.state();
		
		buttonPhoto.read();
		buttonSlider.read();
		potentiometer.read();
		voltageBattery.read();
		
		filterVoltageBattery.set (Math::curve (0, voltageBattery.value, 1023, 0, 30, 0));
		hysteresisVoltageBattery.set (filterVoltageBattery.value * 10.0);
		
		if ((float)(hysteresisVoltageBattery.value) / 10.0 < SETUP_LOW_VOLTAGE_BATTERY)
		{
			lowVoltageBattery = true;
		}
		else
		{
			lowVoltageBattery = false;
		}
		
		if (buttonSlider.bistable == false && startSlider == false)
		{
			timeStartSlider = time.ms;
			startSlider = true;
			
			if (lowVoltageBattery == false)
			{
				SoundWrite::play (2048, 100);
			}
		}
		else if (buttonSlider.bistable == true && startSlider == true)
		{
			startSlider = false;
			slider = false;
		}
		
		if (buttonPhoto.bistable == true && startPhoto == false)
		{
			timeStartPhoto = time.ms;
			startPhoto = true;
			
			if (lowVoltageBattery == false)
			{
				SoundWrite::play (2048, 100);
			}
		}
		else if (buttonPhoto.bistable == false && startPhoto == true)
		{
			countPhoto = 0;
			
			startPhoto = false;
			photo = false;
		}
		
		if (time.ms >= timeStartSlider + 1000ull && startSlider == true && slider == false)
		{
			slider = true;
		}
		
		if (time.ms >= timeStartPhoto + 1000ull && startPhoto == true && photo == false)
		{
			timePhoto = time.ms;
			photo = true;
		}
		
		filterPotentiometer.set (potentiometer.value);
		
		if (slider == true)
		{
			servo.us (Math::curve (0, filterPotentiometer.value, 1023, SETUP_CENTER_SERVO - SETUP_TRAVEL_SERVO, SETUP_CENTER_SERVO + SETUP_TRAVEL_SERVO, 0));
		}
		else
		{
			servo.us (SETUP_CENTER_SERVO);
		}
		
		if (photo == true)
		{
			if (time.ms >= timePhoto + (unsigned long long)(SETUP_INTERVAL))
			{
				shutter.on();
				
				timeIdle = time.ms;
				timePhoto += SETUP_INTERVAL;
				countPhoto++;
				
				if (countPhoto == SETUP_FPS)
				{
					countPhoto = 0;
					
					if (lowVoltageBattery == false)
					{
						SoundWrite::play (2048, 500);
					}
				}
			}
			else if (time.ms >= timeIdle + 10ull)
			{
				shutter.off();
			}
		}
		else
		{
			shutter.off();
		}
		
		if (lowVoltageBattery == true)
		{
			delayLowVoltageBattery.state();
			
			if (delayLowVoltageBattery.update == true)
			{
				led.toggle();
				SoundWrite::play (2048, 100);
			}
		}
		else
		{
			led.on();
		}
	}
	
	return 0;
}
